@@ -1,8 +1,20 @@
 /*
-    This program can be distributed under the terms of the GNU GPL.
-    See the file COPYING.
+    Copyright 2006,2007,2008 Martin Pärtel <martin.partel@gmail.com>
 
-    Author: Martin Prtel <martin.partel@gmail.com>
+    This file is part of bindfs.
+
+    bindfs is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 2 of the License, or
+    (at your option) any later version.
+
+    bindfs is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with bindfs.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef INC_BINDFS_PERMCHAIN_H
