@@ -18,6 +18,8 @@
 #   along with bindfs.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+require 'ruby18_hacks.rb'
+require 'shellwords' unless $ruby_18_hacks_enabled  # Ruby 1.8 doesn't have shellwords
 require 'fileutils'
 include FileUtils
 
@@ -92,7 +94,9 @@ def testenv(bindfs_args, options = {}, &block)
     options = {
         :title => bindfs_args,
         :valgrind => valgrind_options != nil,
-        :valgrind_opts => valgrind_options
+        :valgrind_opts => valgrind_options,
+        :srcdir_name => 'src',
+        :mntdir_name => 'mnt'
     }.merge(options)
 
     # todo: less repetitive and more careful error handling and cleanup
@@ -100,6 +104,9 @@ def testenv(bindfs_args, options = {}, &block)
     puts "--- #{options[:title]} ---"
     puts "[  #{bindfs_args}  ]"
 
+    srcdir = options[:srcdir_name]
+    mntdir = options[:mntdir_name]
+
     begin
         Dir.mkdir TESTDIR_NAME
     rescue Exception => ex
@@ -108,15 +115,20 @@ def testenv(bindfs_args, options = {}, &block)
 
     begin
         Dir.chdir TESTDIR_NAME
-        Dir.mkdir 'src'
-        Dir.mkdir 'mnt'
+        Dir.mkdir srcdir
+        Dir.mkdir mntdir
     rescue Exception => ex
         fail!("ERROR preparing testdir at #{TESTDIR_NAME}", ex)
     end
 
     bindfs_pid = nil
     begin
-        cmd = "../#{EXECUTABLE_PATH} #{bindfs_args} -f src mnt"
+        extra_args = "-f"
+        # Don't rely on user_allow_other in /etc/fuse.conf.
+        # On FreeBSD it isn't even possible to set that.
+        extra_args += " --no-allow-other" if Process.uid != 0
+
+        cmd = "../#{EXECUTABLE_PATH} #{bindfs_args} #{extra_args} #{Shellwords.escape(srcdir)} #{Shellwords.escape(mntdir)}"
         if options[:valgrind]
             cmd = "valgrind --error-exitcode=100 #{options[:valgrind_opts]} #{cmd}"
         end
@@ -146,7 +158,7 @@ def testenv(bindfs_args, options = {}, &block)
     end
 
     begin
-        unless system(umount_cmd + ' mnt')
+        unless system(umount_cmd + ' ' + Shellwords.escape(mntdir))
             raise Exception.new(umount_cmd + " failed with status #{$?}")
         end
         Process.wait bindfs_pid
