@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 #
-#   Copyright 2006,2007,2008,2009,2010,2012 Martin Pärtel <martin.partel@gmail.com>
+#   Copyright 2006,2007,2008,2009,2010,2012,2019 Martin Pärtel <martin.partel@gmail.com>
 #
 #   This file is part of bindfs.
 #
@@ -23,6 +23,7 @@ $LOAD_PATH << (ENV['srcdir'] || '.')
 
 require 'common.rb'
 require 'etc'
+require 'tempfile'
 
 include Errno
 
@@ -66,6 +67,20 @@ testenv("-u nobody -g #{nobody_group}") do
     assert { File.stat('mnt/file').gid == nobody_gid }
 end
 
+testenv("-u #{nobody_uid} -g #{nobody_gid}", :title => "numeric UID and GID") do
+    touch('src/file')
+
+    assert { File.stat('mnt/file').uid == nobody_uid }
+    assert { File.stat('mnt/file').gid == nobody_gid }
+end
+
+testenv("-u 55544 -g 55566", :title => "nonexistent UID and GID") do
+    touch('src/file')
+
+    assert { File.stat('mnt/file').uid == 55544 }
+    assert { File.stat('mnt/file').gid == 55566 }
+end
+
 testenv("-p 0600:u+D") do
     touch('src/file')
     chmod(0777, 'src/file')
@@ -303,8 +318,11 @@ end
 testenv("--rename-deny") do
     touch('src/file')
     mkdir('src/dir')
-    assert_exception(EPERM) { mv('mnt/file', 'mnt/file2') }
-    assert_exception(EPERM) { mv('mnt/dir', 'mnt/dir2') }
+    # We don't use FileUtils.mv because it was changed in Ruby 2.7.1 to
+    # fall back to copying on EPERM:
+    # https://github.com/ruby/ruby/commit/7d3d8e79fe9cc9f21cd4341f0a6fb2e6306688fd
+    assert_exception(EPERM) { File.rename('mnt/file', 'mnt/file2') }
+    assert_exception(EPERM) { File.rename('mnt/dir', 'mnt/dir2') }
 end
 
 root_testenv("--map=nobody/root:@#{nobody_group}/@#{root_group}") do
@@ -378,6 +396,26 @@ root_testenv("--map=0/1:@0/@1", :title => "--map and chown/chgrp") do
     assert { File.stat('mnt/file1').gid == 1 }
 end
 
+Tempfile.create('passwdfile') do |passwd_file|
+    Tempfile.create('groupfile') do |group_file|
+        passwd_file.puts("nobody:x:123:456:,,,:/tmp:/bin/false")
+        passwd_file.flush
+        group_file.write("#{nobody_group}:x:789")
+        group_file.flush
+        root_testenv("--map-passwd=#{Shellwords.escape(passwd_file.path)} --map-group=#{Shellwords.escape(group_file.path)}") do
+            touch('src/file1')
+            chown(123, 789, 'src/file1')
+            assert { File.stat('mnt/file1').uid == nobody_uid }
+            assert { File.stat('mnt/file1').gid == nobody_gid }
+
+            touch('src/file2')
+            chown(nobody_uid, nobody_gid, 'mnt/file2')
+            assert { File.stat('src/file2').uid == 123 }
+            assert { File.stat('src/file2').gid == 789 }
+        end
+    end
+end
+
 root_testenv("--uid-offset=2") do
     touch('src/file')
     chown(1, nil, 'src/file')
